
# q-expansion of newform 3465.2.a.bb, downloaded from the LMFDB on 18 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3465
weight = 2
poly_data = [2, -4, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-3, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, -1, 0], [0, 0, 0], [-1, 0, 0], [-1, 0, 0], [-1, 0, 0], [3, -1, 1], [-2, -2, 1], [0, -2, -1], [2, 0, 1], [-3, -1, 0], [-1, 1, 1], [3, -1, -3], [-1, -1, 3], [1, -3, 2], [-1, 1, 1], [0, 0, -1], [5, -3, 0], [2, -2, 3], [0, 0, -2], [7, -3, 3], [4, 2, 0], [3, -7, -1], [-4, 6, -3], [3, 1, -4], [5, -5, 6], [-2, -4, 4], [3, -1, 0], [7, -3, 5], [1, 1, -5], [2, 6, -1], [9, -3, 4], [5, -1, -5], [0, 10, -4], [-6, -2, -4], [-8, 2, -6], [12, 0, 2], [5, 3, -4], [-1, 5, 5], [-14, 2, 0], [-8, 6, -4], [7, 1, -1], [2, 4, -4], [4, -4, -4], [4, -2, -2], [4, 2, 0], [-8, 8, -4], [6, 2, 4], [-5, 11, -6], [8, -10, -5], [-21, -1, 1], [-1, -9, -1], [9, 5, 0], [-2, 8, 2], [0, 4, -4], [3, 3, -1], [6, 10, -2], [-1, 13, -2], [-10, 4, 5], [2, 8, -6], [2, 8, 2], [5, 7, -1], [4, -16, 1], [18, -2, -4], [-2, 2, 6], [1, 7, -6], [14, -8, 2], [-14, 4, 5], [-7, 7, -6], [18, -6, 6], [0, 4, 3], [-16, -2, 6], [13, -11, -2], [15, -1, 10], [1, -1, 8], [4, -2, 5], [9, 3, 9], [11, 11, -7], [18, -8, -2], [9, -3, 1], [2, 4, 10], [5, -3, 4], [2, -6, 9], [-2, -14, 2], [4, -2, -10], [2, 8, -3], [2, 6, -8], [11, 11, -1], [9, -1, 10], [0, -6, 6], [14, -2, 12], [34, 2, -2], [-11, 11, 3], [10, 18, -4], [9, -11, 14], [6, 0, 5], [-10, 0, -13], [-5, -7, 6], [-13, 1, -4], [-6, 2, -14], [17, 1, -5], [-3, 17, -14], [-12, -18, 6], [-24, 0, -2], [-9, 5, 12], [-27, -5, -3], [2, 0, 0], [11, 5, 3], [-2, 12, -6], [-10, 6, -8], [-20, -12, 3], [14, -18, -4], [26, -4, 4], [6, -20, 2], [20, -4, -2], [4, -6, 1], [-2, 8, -4], [13, -3, 12], [-6, -26, 10], [16, -4, 5], [-1, 15, -12], [3, -1, -11], [5, 11, -2], [-2, 2, 1], [24, -4, 4], [0, 0, 6], [-5, -3, 4], [16, -8, -11], [13, 9, 0], [-27, 5, 2], [0, 22, -6], [-6, 14, -8], [6, -2, -12], [-6, -8, 5], [-16, 6, 6], [2, -4, 0], [4, 12, -1], [-32, 2, 6], [-36, 0, -4], [0, 10, -2], [4, -14, 18], [-4, -16, 0], [-7, 11, -20], [-15, 15, 3], [6, 6, 2], [-16, -2, 6], [11, -1, 4], [-5, -5, -5], [-18, 16, 4], [5, -1, 19], [10, -4, -5], [11, -3, 14], [13, 7, 6], [-18, 14, -22], [32, -14, -5], [-33, 5, 1], [-14, 6, -6], [20, -8, -6], [2, 0, -6], [-11, -19, 9], [-13, 11, 9], [40, -2, -1], [-22, -4, -4], [-3, 13, 10], [-7, 5, -10], [12, -8, -9], [-2, -2, 0], [-8, 6, -7], [-6, -4, -4], [3, -1, -1], [-11, 13, -15], [13, 17, 2], [3, -9, 21], [49, -3, 4], [19, -11, 0], [2, -14, -12], [24, -10, -6], [-35, -5, 9], [-6, -12, -10], [-22, -6, -8], [-14, -24, 14], [-20, 4, -12], [2, -6, 10], [-12, -14, 2], [-8, -20, 15], [18, 12, -1], [5, 15, -8], [-13, 9, 4], [19, -21, 12], [7, -17, 5], [1, 27, -7], [14, -12, -12], [-20, 4, 2], [-22, 22, -12], [-13, 13, 8], [17, 7, -3], [-10, -12, 6], [-18, 16, -18], [-5, -9, -15], [-4, -4, -3], [-22, 10, -8], [-9, 17, -8], [-44, -2, -5], [25, -9, 8], [-16, -2, 4], [-25, -1, 4], [-18, 0, -2], [35, -11, 1], [36, -12, 2], [9, 7, -8], [-23, 19, -5], [57, -5, 0], [33, 3, -12], [30, -6, -14], [3, 37, -9], [-28, 4, -4], [0, -22, -6], [-9, -25, 18], [-4, 18, -22], [46, -10, 4], [0, 8, -1], [-2, 20, 2], [20, 0, 6], [-47, 5, 9], [21, -13, -1], [-30, 0, -9], [-51, 9, 1], [-19, 5, -5], [41, 3, -7], [-10, 26, 6], [11, -1, 22], [51, -9, 5], [-24, 0, 18], [13, 7, 1], [51, -7, 16], [-13, 23, 2], [8, -14, -5], [-37, -9, -1], [8, -18, 12], [14, 2, 16], [10, -2, -2], [32, 0, -6], [-8, 26, 9], [30, -2, -16], [-18, 30, -15], [2, 6, -9], [3, -3, 1], [-12, 12, 12], [-41, -1, 8], [-3, 3, 13], [6, 18, -2], [42, -4, 0], [-15, 5, -15], [-4, 20, -24], [-39, 13, -21], [-18, 10, 9], [-33, -9, 0], [0, 24, -11], [-1, 33, -21], [-1, 15, -7], [20, -10, 6], [-9, -1, -18], [48, -4, -4], [0, 32, -15], [21, 17, 5], [-1, -17, -1], [-7, 7, -19], [-2, 16, -10], [19, -19, 14], [9, 25, 0], [-47, 9, -15], [-14, 12, -28], [-8, -16, 10], [-7, 11, 4], [47, 9, -3], [-51, 1, 7], [30, -18, -16], [42, 16, -9], [-2, -4, 2], [1, 13, -17], [0, -36, 20], [-22, -8, -3], [10, -6, -16], [8, -8, 12], [-18, -14, 3], [61, -1, -9], [15, -7, -19], [21, -19, 13], [14, 8, 6], [24, -4, 10], [-9, -25, 7], [-30, -4, 10], [37, 3, 17], [17, 21, -19], [28, 16, -4], [-22, 12, -16], [22, -24, 6], [2, 6, 2], [7, -29, -5], [8, -36, -4], [-16, 28, -8], [-32, 18, -18], [-37, 15, -29], [21, 7, 13], [-26, -8, 5], [-26, 12, -5], [-9, 19, -17], [44, -26, -15], [39, -9, 7], [29, -1, 9], [35, -19, 38], [-12, 16, -6], [-21, -33, 7], [17, 15, -12], [11, -19, 15], [-21, 29, 5], [-30, 10, -1], [-46, 18, -26], [11, -1, -24], [20, -34, 6], [-13, 9, 12], [40, -12, -8], [29, -19, -19], [-26, 4, 0], [-27, 19, 15], [-32, 10, 12], [-12, 34, -4], [28, 28, -18], [27, -29, 24], [67, -11, 1], [13, 13, 5], [35, -17, -15], [-28, 26, 0], [-26, 8, 3], [-40, 20, 20], [-22, -18, -4], [-32, -20, 16], [-30, 6, 7], [30, -4, 10], [-39, -23, 13], [49, -13, -11], [31, -17, -13], [14, -10, 9], [11, 7, -13], [76, -8, -2], [0, -36, 21], [19, -21, 0], [51, 3, -13], [-13, 29, -25], [16, 24, 4], [-35, -23, 7], [26, -28, 13], [-8, -26, 24], [23, -9, 5], [-13, 29, 13], [-26, -4, 10], [2, 30, -1], [-58, 6, -20], [-35, -1, -5], [-72, -2, 2], [-6, -8, -15], [-13, 37, -14], [26, 4, 16], [-2, -2, 4], [7, -13, 10], [7, -13, 10], [-34, 20, 8], [2, 2, 15], [3, -41, 12], [-3, -15, 27], [6, -42, 22], [10, -14, -14], [38, 18, -18], [-17, 33, 0], [62, -16, 15], [15, 3, 13], [-6, -2, 4], [-17, -9, 22], [-50, 0, -10], [-11, 21, 11], [-19, -7, -9], [-43, -13, 2], [66, 10, -7], [48, 0, -12], [-1, 19, 13], [10, 36, -27], [-1, -31, 13], [-22, 8, -27], [11, 13, 2], [-22, 2, -14], [-10, -6, 2], [12, 4, 15], [8, 18, 2], [-21, 13, -17], [57, -7, 20], [8, 28, 2], [39, -7, -8], [-50, 22, 16], [12, -22, -22], [-22, -12, 25], [-47, 15, -10], [-26, 2, 11], [-2, 18, -25], [23, -1, 22], [-43, 1, -15], [-25, -25, 9], [66, -12, 15], [-37, 35, 19], [-26, -8, 0], [39, -13, -14], [-21, -1, 10], [-50, 0, 14], [-27, -23, 11], [4, -12, 30], [-17, 9, -29], [24, -6, 14], [-25, 9, 4], [15, 19, 8], [7, -17, 18], [-25, -17, -5], [-22, 18, 12], [-37, 31, -13], [-40, 6, 10], [38, -12, 29], [-59, -13, -3], [33, 31, -10], [3, -25, 27], [-14, 0, -17], [65, 11, -6], [52, 4, -14], [11, 13, -19]]
